/*
 *  TOPPERS PROJECT Home Network Working Group Software
 * 
 *  Copyright (C) 2014-2019 Cores Co., Ltd. Japan
 * 
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 */

/* 
 *  サンプルプログラム(1)の本体
 */

#include "shellif.h"
#include <kernel.h>
#include <stdlib.h>
//#include <string.h>
#include "t_stdlib.h"
#include "syssvc/serial.h"
#include "syssvc/syslog.h"
#include "kernel_cfg.h"
#include "diskio.h"
#include "sdfs.h"
#include "ff.h"
#include "tlsf.h"
#include "util/ntstdio.h"

#define SD_CARD_LOG_LEVEL LOG_INFO

#define SIO_PORTID 1
extern ntstdio_t ntstdio;

static tlsf_t sys_tlsf;
static pool_t sys_pool;

uint8_t shell_heap[64 * 1024];
FATFS RomDisk;

gpio_t ins;
sdfs_t sdfs;
#define WP() false

typedef struct SD {
	DSTATUS dst;
	BYTE type;
	FATFS FatFs;
} SD;
SD Sd;

void sys_init(void);
bool_t romdisk_init();
bool_t SD_begin();

int mruby_arduino_init()
{
	int result = -1;

	sys_init();

	/* SD_CD */
	gpio_init_in(&ins, P7_8);

	/* SDカードを初期化 */
	sdfs_init(&sdfs, P8_5, P8_6, P8_3, P8_4, "sd");

	result = SD_begin() ? 0 : -1;
	if (result == 0) {
		ntstdio_printf(&ntstdio, "SD card (1:) OK!\n");
	}
	else {
		ntstdio_printf(&ntstdio, "SD card (1:) NG!\n");
	}
	sta_cyc(SDFS_CYC);

	if (romdisk_init())
		result = 0;

	if (result == 0) {
		ntstdio_printf(&ntstdio, "ROM disk (0:) OK!\n");
	}
	else {
		ntstdio_printf(&ntstdio, "ROM disk (0:) NG!\n");
	}

	/* uploadディレクトリを作成しておく */
	f_mkdir("1:/upload");

	serial_ctl_por(SIO_PORTID, IOCTL_FCSND | IOCTL_FCRCV);

	return result;
}

void sys_init(void)
{
	sys_tlsf = tlsf_create(&shell_heap);
	if (sys_tlsf == NULL)
		return;

	sys_pool = tlsf_add_pool(sys_tlsf, ((uint8_t *)&shell_heap) + tlsf_size(), sizeof(shell_heap) - tlsf_size());
}

void sys_fini(void)
{
	tlsf_destroy(sys_tlsf);
}

void *sys_malloc(size_t size)
{
	return tlsf_malloc(sys_tlsf, size);
}

void *sys_calloc(size_t size, size_t count)
{
	void *result = tlsf_malloc(sys_tlsf, count * size);
	memset(result, 0, count * size);
	return result;
}

void *sys_realloc(void *ptr, size_t size)
{
	return tlsf_realloc(sys_tlsf, ptr, size);
}

void sys_free(void *ptr)
{
	tlsf_free(sys_tlsf, ptr);
}

bool_t romdisk_init()
{
	DSTATUS dst;
	FRESULT res;

	if (RomDisk.fs_type != 0)
		return true;

	if ((dst = ramdisk_initialize()) != RES_OK) {
		return false;
	}

	if ((res = f_mount(&RomDisk, "0:", 1)) != FR_OK) {
		return false;
	}

	return true;
}

bool_t SD_begin()
{
	DSTATUS dst;
	FRESULT res;
	BYTE pdrv = 1, type;

	if (Sd.FatFs.fs_type != 0)
		return true;

	if ((dst = disk_initialize(pdrv)) != RES_OK) {
		return false;
	}

	if ((dst = disk_ioctl(pdrv, MMC_GET_TYPE, &type)) != RES_OK) {
		Sd.dst = dst;
		Sd.type = 0;
	}
	else {
		Sd.dst = RES_OK;
		Sd.type = type;
	}

	if ((res = f_mount(&Sd.FatFs, "1:", 1)) != FR_OK) {
		return false;
	}

	return true;
}

DSTATUS disk_initialize(BYTE pdrv)
{
	switch (pdrv) {
	case 0:
		return ramdisk_initialize();
	case 1:
		return sdfs_initialize(&sdfs);
	}
	return STA_NOINIT;
}

DSTATUS disk_status(BYTE pdrv)
{
	switch (pdrv) {
	case 0:
		return ramdisk_get_status();
	case 1:
		return sdfs_status(&sdfs);
	}
	return STA_NOINIT;
}

DRESULT disk_read(BYTE pdrv, BYTE *buff, DWORD sector, UINT count)
{
	switch (pdrv) {
	case 0:
		return ramdisk_read(buff, sector, count);
	case 1:
		return sdfs_read(&sdfs, buff, sector, count);
	}
	return RES_PARERR;
}

DRESULT disk_write(BYTE pdrv, const BYTE *buff, DWORD sector, UINT count)
{
	switch (pdrv) {
	case 0:
		return ramdisk_write(buff, sector, count);
	case 1:
		return sdfs_write(&sdfs, buff, sector, count);
	}
	return RES_PARERR;
}

DRESULT disk_ioctl(BYTE pdrv, BYTE cmd, void *buff)
{
	switch (pdrv) {
	case 0:
		return RES_PARERR; /* ramdisk_ioctl(cmd, buff); */
	case 1:
		switch(cmd) {
		case CTRL_SYNC:
			return sdfs_sync(&sdfs);
		case GET_SECTOR_COUNT:
			return sdfs_sectors(&sdfs);
		case GET_BLOCK_SIZE:
			*((DWORD*)buff) = 1; // default when not known
			return RES_OK;
		}
	}
	return RES_PARERR;
}

void sdfs_cychdr(intptr_t exinf)
{
	BYTE s;

	s = sdfs._is_initialized;
	if (WP())		/* Write protected */
		s |= STA_PROTECT;
	else		/* Write enabled */
		s &= ~STA_PROTECT;
	if (gpio_read(&ins))	/* Card is in socket */
		s &= ~STA_NODISK;
	else		/* Socket empty */
		s |= (STA_NODISK | STA_NOINIT);
	sdfs._is_initialized = s;
}

void sdfs_debug(const char *fmt, ...)
{
	va_list arp;

	va_start(arp, fmt);
	syslog(SD_CARD_LOG_LEVEL, fmt, arp);
	va_end(arp);
}

int ff_cre_syncobj(BYTE vol, FF_SYNC_t* sobj)
{
	return 1;
}

int ff_req_grant(FF_SYNC_t sobj)
{
	ER ret;
	ret = wai_sem(SEM_FILESYSTEM);
	return ret == E_OK;
}

void ff_rel_grant(FF_SYNC_t sobj)
{
	sig_sem(SEM_FILESYSTEM);
}

int ff_del_syncobj(FF_SYNC_t sobj)
{
	return 1;
}

void* ff_memalloc (UINT msize)
{
	return sys_malloc(msize);
}

void ff_memfree (void* mblock)
{
	sys_free(mblock);
}

DWORD get_fattime(void)
{
	time_t temp;
	struct tm _tm;

	temp = rtc_read();
	gmtime_r(&temp, &_tm);

	return	  ((DWORD)(_tm.tm_year - 1980) << 25)
		| ((DWORD)_tm.tm_mon << 21)
		| ((DWORD)_tm.tm_mday << 16)
		| ((DWORD)_tm.tm_hour << 11)
		| ((DWORD)_tm.tm_min << 5)
		| ((DWORD)_tm.tm_sec >> 1);
}
